%--------------------------------------------------------------------------
% function X_nlin = GenRESET_Regressors(k,q,X_lin,n_hyp,endn,ffn) 
%
% m.file to generate the regressors required to simulate the RESET test
% statistics:
%
% Input:     k ... Number of I(1) regressors
%            q ... Power: q=2: all squares and cross-products,...
%        X_lin ... R^{T \times k} I(1) regressors
%        n_hyp ... number of all parameters (including deterministics and linear terms)
%         endn ... contains number of different terms up to the (r-1)th power
%          ffn ... number of different terms only! for the (r-1)th power
%
% Output:  X_nlin ... R^{T \times dim(X_nlin)} powers of all variables, all cross-products
%                     between two variables (no multiproducts)
%
% OS, December 2013
%--------------------------------------------------------------------------
function X_nlin = GenRESET_Regressors(k,q,X_lin,n_hyp,endn,ffn) 

%% Generation of powers and cross-products:
% working for all q and k

% n_hyp = nchoosek(q+k,k);
n_hyp = n_hyp-k-1;
T = length(X_lin);
X_nlin = zeros(T,n_hyp);

% additional index
t1 = 0;
t2 = 0;

if q > 1 
    % q=2 (build up matrix X_nlin)
    for i = 1:k
        for j = i:k
            t1 = t1 + 1;
            X_nlin(:,t1) = X_lin(:,i).*X_lin(:,j);
        end;
    end;

    % higher values of q
    if q > 2
    for r = 3:q
        en = endn(r); 
        %en = nchoosek(r-1+k,k)-1-k; % different terms up to the (r-1)th power
        for j = 1:k
            t2 = t2 + 1;
            ff = ffn(t2);
            %ff = nchoosek(r-1+k-j,r-1);
            for i = (1+en-ff):en
                t1 = t1 + 1;                
                X_nlin(:,t1) = X_nlin(:,i).*X_lin(:,j); 
            end;
        end;
    end;
    end;
end;