%-------------------------------------------------------------------------
% function test_decis = MoreSpecTest_Size(testval,kern,alphavec,p,k,q,b)
%
% Function to evaluate whether Wald-test is rejected at percentiles
% alphavec when using critical values from asymptotic tables
%
% 
% Input: testval ... Value of test statistic (vector in R^{nr. of tests
%                    \times 1})
%           kern ... Kernel function (usual abbreviation)
%       alphavec ... Vector in R^{numAlpha \times 1} with critical values
%                    considered in set: 2 times - 0.01, 0.025, 0.05
%          p     ... specifies determinstics
%          k     ... number of I(1) regressors
%          q     ... highest order for RESET test
%          b     ... specifies the bandwidth chosen,
%                    in the range (0,1]
%
% Output: 
%    test_decis ... Vector in R^{numAlpha \times 1} with rejection
%                   frequencies
%                   (0 if no rejection and 1 if rejection; per test)
%
% OS, 2013
%--------------------------------------------------------------------------
function test_decis = MoreSpecTest_Size(crittable,testval,kern,alphavec,p,k,q,b)

% Select the row corresponding to the test chosen: selection is equal to 
critrow = crittable(max(floor(50*b),1),:);
[numA,sizeA2] = size(alphavec);
[numTest,SizeT2] = size(testval);
test_decis = zeros(numA,numTest);

for j = 1:numA; % Loop over different alpha-values

    
    % Check the used perc-vec specifications to use proper columns: 
    if alphavec(j,1) == 0.01; % Choice of critical values for spec. alpha        
        vval = critrow(9);
    elseif alphavec(j,1) == 0.025;
        vval = critrow(8);        
    elseif alphavec(j,1) == 0.05;
        vval = critrow(7);
    elseif alphavec(j,1) == 0.1;
        vval = critrow(6);
    elseif alphavec(j,1) == 0.5;
        vval = critrow(5);
    elseif alphavec(j,1) == 0.9;
        vval = critrow(4);
    elseif alphavec(j,1) == 0.95;
        vval = critrow(3);
    elseif alphavec(j,1) == 0.975;
        vval = critrow(2);
    elseif alphavec(j,1) == 0.99;
        vval = critrow(1);        
    end;
    
    % Rejection decision:
    test_decis(j,:) = (testval > vval);
    
end;

test_decis = mean(test_decis,2); % Column vector
