%-------------------------------------------------------------------------
% function [rejprob,rejprob_cor] = RESET(T,p,k,q,b,kern,simnum,percvec,coeffvec);
%
% Reset estimation
%
% Input:   T     ... length of data period
%          p     ... specifies determinstics
%                    -1 ... no deterministics
%                     0 ... only constant
%                     p ... constant + deterministics with
%                           highest order p, if p > 1
%          k     ... number of I(1) regressors
%          q     ... highest order for RESET test
%          b     ... specifies the bandwidth chosen,
%                    either a number between 0 and 1
%                    or Strings('And91','NW','NW_fake')
%          kern  ... Specifies chosen kernel function:
%                tr ... Truncated
%                ba ... Bartlett
%                pa ... Parzen
%                bo ... Bohman
%                da ... Daniell
%                qs ... Quadratic Spectral
%          simnum is the number of simulations done
%          percvec  ... list of quantiles
%          coeffvec is an R^{8 \times 1} coefficient vector!
%
%
% Returns the rejection probabilities, one time asymptotic and one time size
% corrected power
%
% OS, December 2013
%-------------------------------------------------------------------------
function [rejprob,rejprob_cor] = RESET(T,p,k,q,bb,kern,simnum,percvec,coeffvec)

% Values for rho parameters:
rho1vec = [0 0.3 0.6 0.8]; % as for FM-CPR
rho2vec = rho1vec;

[numP,~] = size(percvec);

% Set quantiles:
alpha = percvec;

% Initialize matrix with rejection probabilities:
    rejprob = zeros(numP,4);
    rejprob_cor = zeros(numP,4);

% coefficient vector under null
n_hyp = p+nchoosek(q+k,k); % number of all parameters
th0 = zeros(n_hyp,1);
% coefficients for det- and lin-terms
th0(1:3) = [3,1,1];

% % % % % % % % % % % % % % % % % % % %

[ffn,endn] = PrepGenReg(q,k);

statRESET = zeros(simnum,1);
size_decis = zeros(simnum,numP);

% Deterministic regressors: Not generated within IMOLS_NL
D = poly_regressors(T,p);  

% Call of weight vector generating function QF_weights:
if strcmp(bb,'And91')==0 && strcmp(bb,'NW')==0 && strcmp(bb,'NWT')==0 && strcmp(bb,'And91_datadep')==0 && strcmp(bb,'NW_datadep')==0 && strcmp(bb,'NWT_datadep')==0
    bandw = bb*T; % only for fixed b !!!
    b = bb;
    % done to allow for faster calculations
    weightmat = QF_weights(T,bandw,kern);
end

% Selection of critical value table:
    
% Load the proper table and store it in crittable:
dotm = '.mat';
sizestring = strcat(['RESET_',kern,'_deter_',num2str(p),'_k_',num2str(k),'_q_',num2str(q)]);
sizestring2 = strcat(['RESET_',kern,'_deter_',num2str(p),'_k_',num2str(k),'_q_',num2str(q),dotm]);
load( fullfile('RESET_Code','RESET_KritischeWerte',sizestring2), sizestring)
%eval(sprintf(['load ..\\RESET_Code\\RESET_KritischeWerte\\RESET_',kern,'_deter_',num2str(p),'_k_',num2str(k),'_q_',num2str(q)]));
eval(sprintf(['critsize = RESET_',kern,'_deter_',num2str(p),'_k_',num2str(k),'_q_',num2str(q),';']));

% Selection of critical value table:

if strcmp(bb,'And91')==0 && strcmp(bb,'NW')==0 && strcmp(bb,'NWT')==0 && strcmp(bb,'And91_datadep')==0 && strcmp(bb,'NW_datadep')==0 && strcmp(bb,'NWT_datadep')==0
    
    % Load the proper table and store it in crittable:
    powerstring = strcat(['empcritval_','T_',num2str(T),'_',kern,'_deter_',num2str(p),'_k_',num2str(k),'_q_',num2str(q)]);
    powerstring2 = strcat(['empcritval_','T_',num2str(T),'_',kern,'_deter_',num2str(p),'_k_',num2str(k),'_q_',num2str(q),dotm]);
    load( fullfile('RESET_Code','results',powerstring2), powerstring)
    %eval(sprintf(['load ..\\RESET_Code\\results\\empcritval_','T_',num2str(T),'_',kern,'_deter_',num2str(p),'_k_',num2str(k),'_q_',num2str(q)]));
    eval(sprintf(['critpower = empcritval_','T_',num2str(T),'_',kern,'_deter_',num2str(p),'_k_',num2str(k),'_q_',num2str(q),';']));

else
    
    % Load the proper table and store it in crittable:
    powerstring = strcat(['empcritval_',bb,'_T_',num2str(T),'_',kern,'_deter_',num2str(p),'_k_',num2str(k),'_q_',num2str(q)]);
    powerstring2 = strcat(['empcritval_',bb,'_T_',num2str(T),'_',kern,'_deter_',num2str(p),'_k_',num2str(k),'_q_',num2str(q),dotm]);
    load( fullfile('RESET_Code','results',powerstring2), powerstring)
    %eval(sprintf(['load ..\\RESET_Code\\results\\empcritval_',bb,'_T_',num2str(T),'_',kern,'_deter_',num2str(p),'_k_',num2str(k),'_q_',num2str(q)]));
    eval(sprintf(['critpower = empcritval_',bb,'_T_',num2str(T),'_',kern,'_deter_',num2str(p),'_k_',num2str(k),'_q_',num2str(q),';']));
    
end   


for rho1set = 1:4; % Loop around values of rho1:
    rho1 = rho1vec(rho1set);
    rho2 = rho2vec(rho1set);

    % Setting the seed: IT IS OF UTMOST IMPORTANCE FOR FIXED-B TO HAVE SAME
    % RANDOM NUMBERS THROUGHOUT - I.E. FOR ALL VALUES OF B
    % rng(T); % new method, not working in too old Matlab releases
    randn('state',T); % this can not be used in future releases later than Matlab 2013b

    
    for simloop = 1:simnum;         % Loop over replications            
        
        % Generation of test process:
        [y,X_lin] = sim_TestNLdata(T,coeffvec,rho1,rho2);                     
            
        % Generation of powers of x:
        X_nlin = GenRESET_Regressors(k,q,X_lin,n_hyp,endn,ffn);
            
        % Call of procedure IMOLSNL: **********************************
        [th1,~,V,~,u2] = IMOLS_NL(y,D,X_lin,X_nlin,1,3,1); 
                     
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %             
        deltau(1,1) = u2(1);
        deltau(2:T,1) = diff(u2);
        
        
        % Calculate Oudotv from OLS (only for standard inference)
        Xmat = [D X_lin X_nlin];
        theta = Xmat\y;
        u = y - Xmat*theta;
        v = diff(X_lin);

        % (1) Constructing LR variance Estimators -> move to seperate fct
        if isequal(bb,'And91')
            bandw = And_HAC91([u(2:end) v],kern);
            b = bandw/T;  
            [Lr,~,~] = lr_var([u(2:end) v],kern, bandw, 0);
            O_udotvT = Lr(1,1)-Lr(1,2:end)*inv(Lr(2:end,2:end))*Lr(2:end,1);
        elseif isequal(bb,'NW') 
            bandw = bwNW([u(2:end) v],kern,0,[]); % use of 0 is essential
            b = bandw/T;              
            [Lr,~,~] = lr_var([u(2:end) v],kern, bandw, 0);
            O_udotvT = Lr(1,1)-Lr(1,2:end)*inv(Lr(2:end,2:end))*Lr(2:end,1);
        elseif isequal(bb,'NWT')
    	    bandw = NW_fake(T);
            [Lr,~,~] = lr_var([u(2:end) v],kern, bandw, 0); 
            O_udotvT = Lr(1,1)-Lr(1,2:end)*inv(Lr(2:end,2:end))*Lr(2:end,1);
        elseif isequal(bb,'And91_datadep')
            bandw = And_HAC91(deltau,kern);
            b = bandw/T;  
            bandw = max(floor(50*b),1)/50*T;
            b = bandw/T;
            [Lr,~,~] = lr_var(deltau,kern, bandw, 0); 
            O_udotvT = Lr;
        elseif isequal(bb,'NW_datadep') 
            bandw = bwNW(deltau,kern,0,[]); % use of 0 is essential
            b = bandw/T;              
            bandw = max(floor(50*b),1)/50*T;
            b = bandw/T;
            [Lr,~,~] = lr_var(deltau,kern, bandw, 0); 
            O_udotvT = Lr;
        elseif isequal(bb,'NWT_datadep')
    	    bandw = NW_fake(T);
            b = bandw/T;
            [Lr,~,~] = lr_var(deltau,kern, bandw, 0); 
            O_udotvT = Lr;
        else
            [Lr,~,~] = lr_var_quad(deltau, 0,weightmat); 
            O_udotvT = Lr;
        end  
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %             
        
        % Calculation of Wald-statistic
        Wald = TestPart_FullNL_H(th1,V,O_udotvT,th0,p,k);        
        if strcmp(bb,'And91')==0 && strcmp(bb,'NW')==0 && strcmp(bb,'NWT')==0
            statRESET(simloop,1) = Wald;
            size_decis(simloop,:) = MoreSpecTest_Size(critsize,Wald,kern,alpha,p,k,q,b);
            power_decis(simloop,:) = MoreSpecTest_Power(critpower,Wald,rho1,rho2,kern,T,alpha,p,k,q,bb);           
        else
            statRESET(simloop,1) = Wald;
            vval = chi2inv(1-alpha,n_hyp-k-p-1); % critical value from chi^2 distribution
            size_decis(simloop,:) = (Wald > vval);
            power_decis(simloop,:) = MoreSpecTest_Power(critpower,Wald,rho1,rho2,kern,T,alpha,p,k,q,bb);            
        end
            
                  
    end;                            % End of loop over replications

    % Processing of result: 
    
    % allocate the values: 
    
    % Set quantiles:
    alpha = percvec;

    % asymptotic/raw: 
    if strcmp(bb,'And91')==0 && strcmp(bb,'NW')==0 && strcmp(bb,'NWT')==0
        rejprob(:,rho1set) = mean(size_decis);
    else
        rejprob(:,rho1set) = mean(size_decis); 
    end    
    
    % size corrected:
    if strcmp(bb,'And91')==0 && strcmp(bb,'NW')==0 && strcmp(bb,'NWT')==0
        rejprob_cor(:,rho1set) = mean(power_decis);
    else
        rejprob_cor(:,rho1set) = mean(power_decis); 
    end     
    
end;  % RHO-loop; 
  