%-------------------------------------------------------------------------
% Control program to steer the simulation for the RESET-tests
% to obtain the rejection probabilities, one time asymptotic
% and one time size corrected power:
% This programme essentially calls the program 
% RESET.m
%
%-------------------------------------------------------------------------

% General control parameters:

% Set your own path here
%cd('D:\papers\Vogelsang\Uni\TUDortmund\RESET_Code\RESET_Code')
path=pwd;
addpath ../RESET_Code/RESET_KritischeWerte;
addpath ../RESET_Code/results;

% define model specifications
% coeffvec = [c x1 x2 x1^2 x2^2 x1*x2 x1^3 extra-fct]';
% special case for only two integrated regressors
% to use more than two integrated regressors coeffvec, the simulation of
% the process and the null hypothesis have to be modified
coeffvec = [3 1 1 0 0 0 0 100]'; % other than null process

%%
for model = 6:6
    if model == 1
        coeffvec = [3 1 1 0.01 0 0 0 0]'; % null + x1^2
    elseif model == 2
        coeffvec = [3 1 1 0.01 0 0.01 0 0]'; % null + x1^2 + x1*x2
    elseif model == 3
        coeffvec = [3 1 1 0.01 0.01 0.01 0 0]'; % null + x1^2 + x2^2 + x1*x2
    elseif model == 4
        coeffvec = [3 1 1 0 0 0.01 0 0]'; % null + x1*x2
    elseif model == 5
        coeffvec = [3 1 1 0 0 0 0.001 0]'; % null + x1^3
    elseif model == 6
        coeffvec = [3 1 1 0 0 0 0 0.5]'; % null + logistic smooth transition
    end
% name results properly, so that they can be distinguished
% for instance use numerical value of model    
%%

% Interval for b-parameter:
% 50 bsteps (for other options move down)
bsteps = 50;

% Vector with percentiles considered:
percvec = [0.01 0.025 0.05 0.1 0.5 0.9 0.95 0.975 0.99]';

% Number of simulations:
%simnum = 100000;
simnum = 10000; % number should be higher than bsteps

% Minimal order trend term:
p_min = 0;
% Maximal order trend term:
p_max = 0;

% Minimal number of integrated regressors:
k_min = 2;
% Maximal number of integrated regressors: 
k_max = 2; % for other values significant changes are needed

% Minimal order of RESET terms:
q_min = 2;
% Maximal order of RESET terms:
q_max = 3;  

% Loop around bandwidth choices: fixed or dynamic
for fod = 1:4;
    if fod == 1;
        b = 0; % fixed bandwidths with number of steps defined in bsteps
    elseif fod == 2;
        b = 'And91';
    elseif fod == 3;
        b = 'NWT';
    elseif fod == 4;
        b = 'NW'; 
    elseif fod == 5;
        b = 'And91_datadep';
    elseif fod == 6;
        b = 'NWT_datadep';
    elseif fod == 7;
        b = 'NW_datadep';       
    end;	


% Loop around sample sizes: 
for tloop = 2:3;     % only up to T = 500
    if tloop == 1;
        T = 50;
    elseif tloop == 2;
        T = 100; % currently only T = 100
    elseif tloop == 3;
        T = 200;
    elseif tloop == 4;
        T = 500;
    elseif tloop == 5;
        T = 1000;
    end;

% Loop over different kernels:
for kern_loop = 1:2;                 % LOOP:KERNELS
    
    % Selection of kernel:
    if kern_loop == 1;
        % Bartlett:
        kern = 'ba';
    elseif kern_loop == 2;
        % Quadratic Spectral:
        kern = 'qs';
    elseif kern_loop == 3;
        % Parzen
        kern = 'pa';
    elseif kern_loop == 4;
        % Bohman
        kern = 'bo';
    elseif kern_loop == 5;
        % Daniell
        kern = 'da';
    end;
    
    
% Loop over different deterministic specifications:
  for det_loop = p_min:p_max;
      
% Loop over number of integrated regressors:
  for k_loop = k_min:k_max;
      
% Loop over "Power" of RESET Tests: i.e. squares, cubes,...
  for q_loop = q_min:q_max;
      
    if b == 0      
        rejprob_cor_all = zeros(bsteps,size(percvec,1),4);
        rejprob_raw_all = zeros(bsteps,size(percvec,1),4);
      
        for b_loop = 1:bsteps;
            [rejprob_raw,rejprob_cor] = RESET(T,det_loop,k_loop,q_loop,(b_loop/bsteps),kern,simnum,percvec,coeffvec);
            rejprob_cor_all(b_loop,:,:) = rejprob_cor;
            rejprob_raw_all(b_loop,:,:) = rejprob_raw;
        end;
      
        % NAMES NOT NEGATIVE:
        if det_loop == -1;
            detX = 999; % Matlab does not accept negative numbers in file names.
        else
            detX = det_loop;
        end;
      
        % SAVE RESULTS (only one alternative tested)
      
        % Size corrected Power:
        eval(sprintf(['RESET_cor_','alt_',num2str(model),'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop),'=rejprob_cor_all;']));
     
        filename_c = ['RESET_cor_','alt_',num2str(model),'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop)];
        varname_c = ['RESET_cor_','alt_',num2str(model),'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop)];
        save(filename_c,varname_c);
      
        % Raw Power:
        eval(sprintf(['RESET_raw_','alt_',num2str(model),'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop),'=rejprob_raw_all;']));
     
        filename_a = ['RESET_raw_','alt_',num2str(model),'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop)];
        varname_a = ['RESET_raw_','alt_',num2str(model),'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop)];
        save(filename_a,varname_a);
    
    else    
        
        [rejprob_raw,rejprob_cor] = RESET(T,det_loop,k_loop,q_loop,b,kern,simnum,percvec,coeffvec);
        rejprob_cor_all(1,:,:) = rejprob_cor;
        rejprob_raw_all(1,:,:) = rejprob_raw;
        
        % NAMES NOT NEGATIVE:
        if det_loop == -1;
          detX = 999; % Matlab does not accept negative numbers in file names.
        else
           detX = det_loop;
        end;        

        % Size corrected Power:
        eval(sprintf(['RESET_cor_',b,'_alt_',num2str(model),'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop),'=rejprob_cor_all;']));
     
        filename_c = ['RESET_cor_',b,'_alt_',num2str(model),'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop)];
        varname_c = ['RESET_cor_',b,'_alt_',num2str(model),'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop)];
        save(filename_c,varname_c);
      
        % Raw Power:
        eval(sprintf(['RESET_raw_',b,'_alt_',num2str(model),'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop),'=rejprob_raw_all;']));
     
        filename_a = ['RESET_raw_',b,'_alt_',num2str(model),'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop)];
        varname_a = ['RESET_raw_',b,'_alt_',num2str(model),'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop)];
        save(filename_a,varname_a);        
		
    end       
   
    
  end;                               % END OF LOOP OVER POWER-ORDER RESET
  end;                               % END OF LOOP OVER NR. of INT.REGS       
  end;                               % END OF LOOP OVER DETERMINISTICS      
end;                                 % END OF LOOP OVER KERNELS:
end % End loop tloop 


end								 % END OF LOOP OVER BANDWIDTH CHOICES

end