%-------------------------------------------------------------------------
% Control program to steer the simulation for the RESET-tests
% to obtain the rejection probabilities, one time asymptotic
% and one time size corrected power:
% This programme essentially calls the program 
% RESET.m
%
%-------------------------------------------------------------------------

% General control parameters:

% Set your own path here
%cd('D:\papers\Vogelsang\Uni\TUDortmund\RESET_Code\RESET_Code')
path=pwd;
addpath ../RESET_Code/RESET_KritischeWerte;
addpath ../RESET_Code/results;

% define model specifications
% coeffvec = [c x1 x2 x1^2 x2^2 x1*x2 x1^3 extra-fct]';
% special case for only two integrated regressors
% to use more than two integrated regressors coeffvec, the simulation of
% the process and the null hypothesis have to be modified
coeffvec = [3 1 1 0 0 0 0 1]'; % other than null process

% Interval for b-parameter:
% 50 bsteps (for other options move down)
bsteps = 50;

% Vector with percentiles considered:
percvec = [0.01 0.025 0.05 0.1 0.5 0.9 0.95 0.975 0.99]';

% Number of simulations:
%simnum = 100000;
simnum = 10000; % number should be higher than bsteps

% Minimal order trend term:
p_min = 0;
% Maximal order trend term:
p_max = 0;

% Minimal number of integrated regressors:
k_min = 2;
% Maximal number of integrated regressors: 
k_max = 2; % for other values significant changes are needed

% Minimal order of RESET terms:
q_min = 2;
% Maximal order of RESET terms:
q_max = 3;  

% Power curve: 
% curve = 1; % YES=1, NO=0
csteps = 20; % number of simulations with different coefficients for power curve



% Loop around bandwidth choices: fixed or dynamic
for fod = 1:1;
    if fod == 1;
        b = 0; % fixed bandwidths with number of steps defined in bsteps
    elseif fod == 2;
        b = 'And91';
    elseif fod == 3;
        b = 'NWT';
    elseif fod == 4;
        b = 'NW'; 
    elseif fod == 5;
        b = 'And91_datadep';
    elseif fod == 6;
        b = 'NWT_datadep';
    elseif fod == 7;
        b = 'NW_datadep';       
    end;	


% Loop around sample sizes: 
for tloop = 2:4;     % only up to T = 500
    if tloop == 1;
        T = 50;
    elseif tloop == 2;
        T = 100; % currently only T = 100
    elseif tloop == 3;
        T = 200;
    elseif tloop == 4;
        T = 500;
    elseif tloop == 5;
        T = 1000;
    end;

% Loop over different kernels:
for kern_loop = 1:2;                 % LOOP:KERNELS
    
    % Selection of kernel:
    if kern_loop == 1;
        % Bartlett:
        kern = 'ba';
    elseif kern_loop == 2;
        % Quadratic Spectral:
        kern = 'qs';
    elseif kern_loop == 3;
        % Parzen
        kern = 'pa';
    elseif kern_loop == 4;
        % Bohman
        kern = 'bo';
    elseif kern_loop == 5;
        % Daniell
        kern = 'da';
    end;
    
    
% Loop over different deterministic specifications:
  for det_loop = p_min:p_max;
      
% Loop over number of integrated regressors:
  for k_loop = k_min:k_max;
      
% Loop over "Power" of RESET Tests: i.e. squares, cubes,...
  for q_loop = q_min:q_max;
      
    if b == 0
        
        rejprob_cor_all = zeros(csteps+1,size(percvec,1),4);
        rejprob_raw_all = zeros(csteps+1,size(percvec,1),4);  
        
        for model = 1:1
            
        bnum = 8; % for instance   
        for b_loop = 1:bnum; % bsteps;
        if b_loop == 1
            bw = 0.02;   
        elseif b_loop == 2
            bw = 0.06;
        elseif b_loop == 3
            bw = 0.1;
        elseif b_loop == 4
            bw = 0.3;
        elseif b_loop == 5
            bw = 0.5;
        elseif b_loop == 6
            bw = 0.7;
        elseif b_loop == 7
            bw = 0.9;         
        elseif b_loop == 8
            bw = 1;
        end
        
        
        for c_loop = 1:(csteps+1)
        if model == 1
            coeffvec = [3 1 1 ((c_loop-1)/csteps)/25 0 0 0 0]';
        elseif model == 2
            coeffvec = [3 1 1 0 0 0 0 ((c_loop-1)/csteps)/1]';
        end				
      
        [rejprob_raw,rejprob_cor] = RESET(T,det_loop,k_loop,q_loop,bw,kern,simnum,percvec,coeffvec);
        rejprob_cor_all(c_loop,:,:) = rejprob_cor;
        rejprob_raw_all(c_loop,:,:) = rejprob_raw;
      
        % NAMES NOT NEGATIVE:
        if det_loop == -1;
            detX = 999; % Matlab does not accept negative numbers in file names.
        else
            detX = det_loop;
        end;
      
        % SAVE RESULTS
        
        % Size corrected Power:
        eval(sprintf(['RESET_cor_curve_',num2str(model),'_b_',num2str(50*bw),'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop),'=rejprob_cor_all;']));
     
        filename_c = ['RESET_cor_curve_',num2str(model),'_b_',num2str(50*bw),'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop)];
        varname_c = ['RESET_cor_curve_',num2str(model),'_b_',num2str(50*bw),'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop)];
        save(filename_c,varname_c);
      
        % Raw Power:
        eval(sprintf(['RESET_raw_curve_',num2str(model),'_b_',num2str(50*bw),'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop),'=rejprob_raw_all;']));
     
        filename_a = ['RESET_raw_curve_',num2str(model),'_b_',num2str(50*bw),'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop)];
        varname_a = ['RESET_raw_curve_',num2str(model),'_b_',num2str(50*bw),'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop)];
        save(filename_a,varname_a); 
        
        end
        
        end; % END B_LOOP     
        
        end % END MODEL LOOP
        
    else     
        % SAVE RESULTS (number of alternatives to be tested)
        
		% Number of different coefficient vectors for power curve with dynamic bandwidths
		
        for model = 1:1
 
        for c_loop = 1:(csteps+1)
        if model == 1
            coeffvec = [3 1 1 ((c_loop-1)/csteps)/25 0 0 0 0]';
        elseif model == 2
            coeffvec = [3 1 1 0 0 0 0 ((c_loop-1)/csteps)/1]';
        end				      

        [rejprob_raw,rejprob_cor] = RESET(T,det_loop,k_loop,q_loop,b,kern,simnum,percvec,coeffvec);
        rejprob_cor_all(c_loop,:,:) = rejprob_cor;
        rejprob_raw_all(c_loop,:,:) = rejprob_raw;
        
        % NAMES NOT NEGATIVE:
        if det_loop == -1;
          detX = 999; % Matlab does not accept negative numbers in file names.
        else
           detX = det_loop;
        end;        

        % Size corrected Power:
        eval(sprintf(['RESET_cor_curve_',num2str(model),'_',b,'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop),'=rejprob_cor_all;']));
     
        filename_c = ['RESET_cor_curve_',num2str(model),'_',b,'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop)];
        varname_c = ['RESET_cor_curve_',num2str(model),'_',b,'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop)];
        save(filename_c,varname_c);
      
        % Raw Power:
        eval(sprintf(['RESET_raw_curve_',num2str(model),'_',b,'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop),'=rejprob_raw_all;']));
     
        filename_a = ['RESET_raw_curve_',num2str(model),'_',b,'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop)];
        varname_a = ['RESET_raw_curve_',num2str(model),'_',b,'_T_',num2str(T),'_',kern,'_deter_',num2str(detX),'_k_',num2str(k_loop),'_q_',num2str(q_loop)];
        save(filename_a,varname_a);  
       
		end                              % END OF LOOP OVER COEFFICIENTS
		
        end                              % END OF MODEL LOOP
    end       
   
    
  end;                               % END OF LOOP OVER POWER-ORDER RESET
  end;                               % END OF LOOP OVER NR. of INT.REGS       
  end;                               % END OF LOOP OVER DETERMINISTICS      
end;                                 % END OF LOOP OVER KERNELS:
end % End loop tloop 


end								 % END OF LOOP OVER BANDWIDTH CHOICES