%----------------------------------------------------------
% function [Omega,Delta,Sigma] = lr_var_quad(u,kern,band,deme%,weightmat)
%
% This function computes the long-run variance Omega,
% the one sided long-run variance Delta (starting with Lag 0) and
% the variance Sigma from an input matrix of residuals.
% in a system of the form:
% REMARK: Delta is only available for scalar case!!!
%       
% Input:   u     ... Residual matrix, in R^{T \times m}
%          kern  ... Specifies chosen kernel function:
%                tr ... Truncated
%                ba ... Bartlett
%                pa ... Parzen
%                bo ... Bohman
%                da ... Daniell
%                qs ... Quadratic Spectral
%          band  ... specifies the bandwidth chosen, integer
%                    in the range 1,...,T
%          deme  ... Demeaning of residuals (1 yes, 0 no)
%      %weightmat ... weight matrix calculated before
% Output:  Omega ... Long-run variance matrix
%          Delta ... One-sided long-run variance matrix
%          Sigma ... Variance matrix
%
% External function: QF_weights;
%
% TV and MW, October 2009.
%----------------------------------------------------------
function [Omega,Delta,Sigma] = lr_var_quad(u,deme,weightmat)
% function [Omega,Delta,Sigma] = lr_var_quad(u,kern,band,deme,weightmat)
% kern and band needed to obtain weightmat

[T,m] = size(u);

% Demeaning residuals (full vector demeaning):
if deme == 1;
    u = u-ones(1,T)*u/T; % this way is faster than using ordinary mean
    % u = u-ones(T,1)*mean(u);
end;

% % Call of weight vector generating function QF_weights:
% weightmat = QF_weights(T,band,kern);
% calculated externely -> allows faster computations for fixed-b

% Variance (scaling only by 1/T): 
Sigma = (u'*u)/T;

% Long-run variance:
Omega = u'*weightmat*u;
Omega = Omega/T;

% One-sided long-run variance:
if m == 1,
    Delta = 0.5*(Omega+Sigma);
elseif m > 1,
    Delta = zeros(m,m);
end;

