%-------------------------------------------------------------------------
% function [y,x] = sim_TestNLdata(T,coeffvec,rho1,rho2,rho3);
%
% Function to generate data:        y = c + x1*beta1 + x2*beta2 x1^2*beta3
%                                     + x2^2*beta4 + x_1x_2*beta5 + x^3*beta6
%                                     + xg*beta7 + u
%                             Delta x = v
%   +  COEFFVEC IS A COLUMN VECTOR
% Coeffvec is an R^{8 \times 1} vector!
% The joint vector [u,v] is given by: (1-rho1L)u = e1 + rho2*e2+rho3*e3
%                                              v1 = e2 + 0.5*e2(-1),
%                                              v2 = e3 + 0.5*e3(-1),
% with e1,e2 standard normally distributed.
% Similar DGP (except not predetermined) as in HW08
%
% Returns the dep. variable y, the matrix of all nondet. regressors.
% Data are returned as column vectors respectively matrices
%
% MW, modifications by OS 2013
%-------------------------------------------------------------------------
function [y,x_lin] = sim_TestNLdata(T,coeffvec,rho1,rho2)

% Starting point with no nonlinear terms:
xs1 = [];
xs2 = [];
x12 = [];
xc1 = [];
xg = [];

% Generation of Random errors e1 and e2 to be standard normal:
                e = randn(T+1,3);                
                e1 = e(:,1);e2 = e(:,2);e3 = e(:,3);

% Generation of error terms u:
u = zeros(T+1,1);

% Lagged error terms:
e2l = e2(1:T);
e3l = e3(1:T);

% Set v's according to model:
v1 = 1*e2(2:end) + 0.5*e2l;
v2 = 1*e3(2:end) + 0.5*e3l;

% Resulting error term:
for j = 2:T+1;
    u(j,1) = rho1*u(j-1,1) + e1(j) + rho2*(e2(j) + e3(j));
end;

u = u(2:end,1);

% X regressor as cumulated v process:
% Linear Part: 
x1 = cumsum(v1);
x2 = cumsum(v2);
x_lin = [x1 x2];

% Squares:
if coeffvec(4)~=0
    xs1 = x1.*x1;
end
if coeffvec(5)~=0
    xs2 = x2.*x2;
end
% cross-product:
if coeffvec(6)~=0
    x12 = x1.*x2;
end

% Cubic:
if coeffvec(7)~=0
    xc1 = x1.^3;
end

% extra function:
if coeffvec(8)~=0
    xg = zeros(T,1);
    c = 0;
    gamma = 1;    % gamma = 1 auch ok
    for i = 1:T
        xg(i,1) = x1(i)/(1+exp(-gamma*(x1(i)-c)));
    end
end

% constant:
cterm = ones(T,1);

% all terms together:
x_all = [cterm x_lin xs1 xs2 x12 xc1 xg];

% only non-zero terms are needed:
coeffvec(coeffvec==0)=[];

% Y as given by function:
y = x_all*coeffvec + u; 